@@ -76,6 +76,8 @@ static void* adr_getter (EContact *contact, EVCardAttribute *attr);
 static void adr_setter (EContact *contact, EVCardAttribute *attr, void *data);
 static void* date_getter (EContact *contact, EVCardAttribute *attr);
 static void date_setter (EContact *contact, EVCardAttribute *attr, void *data);
+static void* cert_getter (EContact *contact, EVCardAttribute *attr);
+static void cert_setter (EContact *contact, EVCardAttribute *attr, void *data);
 
 #define STRING_FIELD(id,vc,n,pn,ro)  { E_CONTACT_FIELD_TYPE_STRING, (id), (vc), (n), (pn), (ro) }
 #define BOOLEAN_FIELD(id,vc,n,pn,ro)  { E_CONTACT_FIELD_TYPE_BOOLEAN, (id), (vc), (n), (pn), (ro) }
@@ -87,6 +89,7 @@ static void date_setter (EContact *contact, EVCardAttribute *attr, void *data);
 #define LIST_ELEM_STR_FIELD(id,vc,n,pn,ro,nm) { E_CONTACT_FIELD_TYPE_LIST_ELEM | E_CONTACT_FIELD_TYPE_SYNTHETIC | E_CONTACT_FIELD_TYPE_STRING, (id), (vc), (n), (pn), (ro), (nm) }
 #define MULTI_ELEM_STR_FIELD(id,vc,n,pn,ro,nm) { E_CONTACT_FIELD_TYPE_MULTI_ELEM | E_CONTACT_FIELD_TYPE_SYNTHETIC | E_CONTACT_FIELD_TYPE_STRING, (id), (vc), (n), (pn), (ro), (nm) }
 #define ATTR_TYPE_STR_FIELD(id,vc,n,pn,ro,at1,nth) { E_CONTACT_FIELD_TYPE_ATTR_TYPE | E_CONTACT_FIELD_TYPE_SYNTHETIC | E_CONTACT_FIELD_TYPE_STRING, (id), (vc), (n), (pn), (ro), (nth), (at1), NULL }
+#define ATTR_TYPE_GETSET_FIELD(id,vc,n,pn,ro,at1,nth,get,set) { E_CONTACT_FIELD_TYPE_ATTR_TYPE | E_CONTACT_FIELD_TYPE_GETSET, (id), (vc), (n), (pn), (ro), (nth), (at1), NULL, (get), (set) }
 #define ATTR2_TYPE_STR_FIELD(id,vc,n,pn,ro,at1,at2,nth) { E_CONTACT_FIELD_TYPE_ATTR_TYPE | E_CONTACT_FIELD_TYPE_SYNTHETIC | E_CONTACT_FIELD_TYPE_STRING, (id), (vc), (n), (pn), (ro), (nth), (at1), (at2) }
 #define ATTR_TYPE_STRUCT_FIELD(id,vc,n,pn,ro,at,get,set,ty) { E_CONTACT_FIELD_TYPE_ATTR_TYPE | E_CONTACT_FIELD_TYPE_SYNTHETIC | E_CONTACT_FIELD_TYPE_GETSET | E_CONTACT_FIELD_TYPE_STRUCT, (id), (vc), (n), (pn), (ro), 0, (at), NULL, (get), (set), (ty) }
 
@@ -115,11 +118,11 @@ static EContactFieldInfo field_info[] = {
 	ATTR_TYPE_STR_FIELD (E_CONTACT_ADDRESS_LABEL_WORK,  EVC_LABEL, "address_label_work",  N_("Work Address Label"),  FALSE, "WORK", 0),
 	ATTR_TYPE_STR_FIELD (E_CONTACT_ADDRESS_LABEL_OTHER, EVC_LABEL, "address_label_other", N_("Other Address Label"), FALSE, "OTHER", 0),
 
-	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_ASSISTANT,    EVC_TEL, "assistant_phone",   N_("Assistant Phone"),  FALSE, "X-EVOLUTION-ASSISTANT", 0),
+	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_ASSISTANT,    EVC_TEL, "assistant_phone",   N_("Assistant Phone"),  FALSE, EVC_X_ASSISTANT, 0),
 	ATTR2_TYPE_STR_FIELD (E_CONTACT_PHONE_BUSINESS,     EVC_TEL, "business_phone",    N_("Business Phone"),   FALSE, "WORK", "VOICE",         0),
 	ATTR2_TYPE_STR_FIELD (E_CONTACT_PHONE_BUSINESS_2,   EVC_TEL, "business_phone_2",  N_("Business Phone 2"), FALSE, "WORK", "VOICE",         1),
 	ATTR2_TYPE_STR_FIELD (E_CONTACT_PHONE_BUSINESS_FAX, EVC_TEL, "business_fax",      N_("Business Fax"),     FALSE, "WORK", "FAX",           0),
-	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_CALLBACK,     EVC_TEL, "callback_phone",    N_("Callback Phone"),   FALSE, "X-EVOLUTION-CALLBACK",  0),
+	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_CALLBACK,     EVC_TEL, "callback_phone",    N_("Callback Phone"),   FALSE, EVC_X_CALLBACK,  0),
 	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_CAR,          EVC_TEL, "car_phone",         N_("Car Phone"),        FALSE, "CAR",                   0),
 	ATTR2_TYPE_STR_FIELD (E_CONTACT_PHONE_COMPANY,      EVC_TEL, "company_phone",     N_("Company Phone"),    FALSE, "WORK", "VOICE",         2),
 	ATTR2_TYPE_STR_FIELD (E_CONTACT_PHONE_HOME,         EVC_TEL, "home_phone",        N_("Home Phone"),       FALSE, "HOME", "VOICE",         0),
@@ -131,9 +134,9 @@ static EContactFieldInfo field_info[] = {
 	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_OTHER_FAX,    EVC_TEL, "other_fax",         N_("Other Fax"),        FALSE, "FAX",                   0), /* XXX */
 	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_PAGER,        EVC_TEL, "pager",             N_("Pager"),            FALSE, "PAGER",                 0),
 	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_PRIMARY,      EVC_TEL, "primary_phone",     N_("Primary Phone"),    FALSE, "PREF",                  0),
-	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_RADIO,        EVC_TEL, "radio",             N_("Radio"),            FALSE, "X-EVOLUTION-RADIO",     0),
-	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_TELEX,        EVC_TEL, "telex",             N_("Telex"),            FALSE, "X-EVOLUTION-TELEX",     0),
-	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_TTYTDD,       EVC_TEL, "tty",               N_("TTY"),              FALSE, "X-EVOLUTION-TTYTDD",    0),
+	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_RADIO,        EVC_TEL, "radio",             N_("Radio"),            FALSE, EVC_X_RADIO,     0),
+	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_TELEX,        EVC_TEL, "telex",             N_("Telex"),            FALSE, EVC_X_TELEX,     0),
+	ATTR_TYPE_STR_FIELD  (E_CONTACT_PHONE_TTYTDD,       EVC_TEL, "tty",               N_("TTY"),              FALSE, EVC_X_TTYTDD,    0),
 	
 	/* Email fields */
 	MULTI_LIST_FIELD     (E_CONTACT_EMAIL,      EVC_EMAIL,        "email",      N_("Email List"),      FALSE),
@@ -168,6 +171,9 @@ static EContactFieldInfo field_info[] = {
 	STRUCT_FIELD    (E_CONTACT_PHOTO, EVC_PHOTO, "photo", N_("Photo"), FALSE, photo_getter, photo_setter, e_contact_photo_get_type),
 	STRUCT_FIELD    (E_CONTACT_LOGO,  EVC_LOGO,  "logo",  N_("Logo"),  FALSE, photo_getter, photo_setter, e_contact_photo_get_type),
 
+	/* Security fields */
+	ATTR_TYPE_STRUCT_FIELD (E_CONTACT_X509_CERT,  EVC_KEY, "x509Cert",  N_("X.509 Certificate"), FALSE, "X509", cert_getter, cert_setter, e_contact_cert_get_type),
+
 	/* Contact categories */
 #if notyet
 	LIST_FIELD      (E_CONTACT_CATEGORY_LIST, EVC_CATEGORIES, "category_list", N_("Category List"), FALSE),
@@ -370,7 +376,6 @@ photo_setter (EContact *contact, EVCardAttribute *attr, void *data)
 						e_vcard_attribute_param_new (EVC_TYPE),
 						image_type);
 
-	printf ("adding photo of type `%s' of length %d\n", image_type, photo->length);
 	e_vcard_attribute_add_value_decoded (attr, photo->data, photo->length);
 }
 
@@ -467,6 +472,7 @@ adr_getter (EContact *contact, EVCardAttribute *attr)
 		GList *p = e_vcard_attribute_get_values (attr);
 		EContactAddress *addr = g_new (EContactAddress, 1);
 
+		addr->address_format = g_strdup ("");
 		addr->po       = g_strdup (p && p->data ? p->data : ""); if (p) p = p->next;
 		addr->ext      = g_strdup (p && p->data ? p->data : ""); if (p) p = p->next;
 		addr->street   = g_strdup (p && p->data ? p->data : ""); if (p) p = p->next;
@@ -522,6 +528,46 @@ date_setter (EContact *contact, EVCardAttribute *attr, void *data)
 
 
 
+static void*
+cert_getter (EContact *contact, EVCardAttribute *attr)
+{
+	if (attr) {
+		/* the certificate is stored in this vcard.  just
+		   return the data */
+		GList *values = e_vcard_attribute_get_values_decoded (attr);
+
+		if (values && values->data) {
+			GString *s = values->data;
+			EContactCert *cert = g_new (EContactCert, 1);
+
+			cert->length = s->len;
+			cert->data = g_malloc (cert->length);
+			memcpy (cert->data, s->str, cert->length);
+
+			return cert;
+		}
+	}
+
+	/* XXX if we stored a fingerprint in the cert we could look it
+	   up via NSS, but that would require the additional NSS dep
+	   here, and we'd have more than one process opening the
+	   certdb, which is bad.  *sigh* */
+}
+
+static void
+cert_setter (EContact *contact, EVCardAttribute *attr, void *data)
+{
+	EContactCert *cert = data;
+
+	e_vcard_attribute_add_param_with_value (attr,
+						e_vcard_attribute_param_new (EVC_ENCODING),
+						"b");
+
+	e_vcard_attribute_add_value_decoded (attr, cert->data, cert->length);
+}
+
+
+
 /* Set_arg handler for the contact */
 static void
 e_contact_set_property (GObject *object,
@@ -619,7 +665,6 @@ e_contact_set_property (GObject *object,
 			int num_left = info->list_elem;
 			GList *attrs = e_vcard_get_attributes (E_VCARD (contact));
 			GList *l;
-			const char *sval = g_value_get_string (value);
 
 			for (l = attrs; l && !found; l = l->next) {
 				const char *name, *group;
@@ -683,10 +728,19 @@ e_contact_set_property (GObject *object,
 										info->attr_type2);
 			}
 
-			if (sval && *sval)
-				e_vcard_attribute_add_value (attr, sval);
-			else
-				e_vcard_remove_attribute (E_VCARD (contact), attr);
+			if (info->t & E_CONTACT_FIELD_TYPE_STRUCT || info->t & E_CONTACT_FIELD_TYPE_GETSET) {
+				void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : g_value_get_pointer (value);
+
+				info->struct_setter (contact, attr, data);
+			}
+			else {
+				const char *sval = g_value_get_string (value);
+
+				if (sval && *sval)
+					e_vcard_attribute_add_value (attr, sval);
+				else
+					e_vcard_remove_attribute (E_VCARD (contact), attr);
+			}
 		}
 		else if (info->t & E_CONTACT_FIELD_TYPE_LIST_ELEM) {
 			EVCardAttribute *attr = e_contact_get_first_attr (contact, info->vcard_field_name);
@@ -723,6 +777,26 @@ e_contact_set_property (GObject *object,
 
 		}
 	}
+	else if (info->t & E_CONTACT_FIELD_TYPE_STRUCT || info->t & E_CONTACT_FIELD_TYPE_GETSET) {
+		EVCardAttribute *attr = e_contact_get_first_attr (contact, info->vcard_field_name);
+		void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : g_value_get_pointer (value);
+
+		if (attr) {
+			printf ("overwriting existing %s\n", info->vcard_field_name);
+			/* remove all existing values and parameters.
+			   the setter will add the correct ones */
+			e_vcard_attribute_remove_values (attr);
+			e_vcard_attribute_remove_params (attr);
+		}
+		else {
+			printf ("adding new %s\n", info->vcard_field_name);
+			attr = e_vcard_attribute_new (NULL, info->vcard_field_name);
+
+			e_vcard_add_attribute (E_VCARD (contact), attr);
+		}
+
+		info->struct_setter (contact, attr, data);
+	}
 	else if (info->t & E_CONTACT_FIELD_TYPE_BOOLEAN) {
 		EVCardAttribute *attr;
 
@@ -759,26 +833,6 @@ e_contact_set_property (GObject *object,
 							  g_value_get_string (value));
 		}
 	}
-	else if (info->t & E_CONTACT_FIELD_TYPE_STRUCT || info->t & E_CONTACT_FIELD_TYPE_GETSET) {
-		EVCardAttribute *attr = e_contact_get_first_attr (contact, info->vcard_field_name);
-		void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : g_value_get_pointer (value);
-
-		if (attr) {
-			printf ("overwriting existing %s\n", info->vcard_field_name);
-			/* remove all existing values and parameters.
-			   the setter will add the correct ones */
-			e_vcard_attribute_remove_values (attr);
-			e_vcard_attribute_remove_params (attr);
-		}
-		else {
-			printf ("adding new %s\n", info->vcard_field_name);
-			attr = e_vcard_attribute_new (NULL, info->vcard_field_name);
-
-			e_vcard_add_attribute (E_VCARD (contact), attr);
-		}
-
-		info->struct_setter (contact, attr, data);
-	}
 	else {
 		g_warning ("unhandled attribute `%s'", info->vcard_field_name);
 	}
@@ -940,6 +994,7 @@ e_contact_get_property (GObject *object,
 	else if (info->t & E_CONTACT_FIELD_TYPE_MULTI_ELEM) {
 		if (info->t & E_CONTACT_FIELD_TYPE_STRING) {
 			GList *attrs, *l;
+			int num_left = info->list_elem;
 
 			attrs = e_vcard_get_attributes (E_VCARD (contact));
 
@@ -952,15 +1007,12 @@ e_contact_get_property (GObject *object,
 
 				/* all the attributes we care about should be in group "" */
 				if ((!group || !*group) && !strcasecmp (name, info->vcard_field_name)) {
-					GList *v;
-					int count;
+					if (num_left-- == 0) {
+						GList *v = e_vcard_attribute_get_values (attr);
 
-					v = e_vcard_attribute_get_values (attr);
-					count = info->list_elem;
-
-					v = g_list_nth (v, info->list_elem);
-
-					g_value_set_string (value, v ? v->data : NULL);
+						g_value_set_string (value, v ? v->data : NULL);
+						break;
+					}
 				}
 			}
 		}
@@ -1431,3 +1483,36 @@ e_contact_address_get_type (void)
 							(GBoxedFreeFunc) e_contact_address_free);
 	return type_id;
 }
+
+void
+e_contact_cert_free (EContactCert *cert)
+{
+	if (!cert)
+		return;
+
+	g_free (cert->data);
+	g_free (cert);
+}
+
+static EContactCert *
+e_contact_cert_copy (EContactCert *cert)
+{
+	EContactCert *cert2 = g_new (EContactCert, 1);
+	cert2->length = cert->length;
+	cert2->data = g_malloc (cert2->length);
+	memcpy (cert2->data, cert->data, cert->length);
+
+	return cert2;
+}
+
+GType
+e_contact_cert_get_type (void)
+{
+	static GType type_id = 0;
+
+	if (!type_id)
+		type_id = g_boxed_type_register_static ("EContactCert",
+							(GBoxedCopyFunc) e_contact_cert_copy,
+							(GBoxedFreeFunc) e_contact_cert_free);
+	return type_id;
+}
