@@ -31,6 +31,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libedataserver/e-xml-hash-utils.h>
 #include <libecal/e-cal-recur.h>
+#include <libecal/e-cal-time-util.h>
 #include <libecal/e-cal-util.h>
 #include <libedata-cal/e-cal-backend-util.h>
 #include <libedata-cal/e-cal-backend-sexp.h>
@@ -317,7 +318,7 @@ e_cal_backend_file_get_alarm_email_address (ECalBackendSync *backend, EDataCal *
 static ECalBackendSyncStatus
 e_cal_backend_file_get_static_capabilities (ECalBackendSync *backend, EDataCal *cal, char **capabilities)
 {
-	*capabilities = CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS;
+	*capabilities = g_strdup (CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS);
 	
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -643,12 +644,11 @@ open_cal (ECalBackendFile *cbfile, const char *uristr)
 	}
 
 	priv->icalcomp = icalcomp;
+	priv->uri = get_uri_string_for_gnome_vfs (E_CAL_BACKEND (cbfile));
 
 	priv->comp_uid_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	scan_vcalendar (cbfile);
 
-	priv->uri = get_uri_string_for_gnome_vfs (E_CAL_BACKEND (cbfile));
-
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -827,7 +827,8 @@ get_uri_string (ECalBackend *backend)
 
 /* Open handler for the file backend */
 static ECalBackendSyncStatus
-e_cal_backend_file_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists)
+e_cal_backend_file_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
+			 const char *username, const char *password)
 {
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
@@ -1259,7 +1260,7 @@ create_user_free_busy (ECalBackendFile *cbfile, const char *address, const char
 	icalcomponent *vfb;
 	icaltimezone *utc_zone;
 	ECalBackendSExp *obj_sexp;
-	char *query;
+	char *query, *iso_start, *iso_end;
 	
 	priv = cbfile->priv;
 
@@ -1282,9 +1283,14 @@ create_user_free_busy (ECalBackendFile *cbfile, const char *address, const char
 	icalcomponent_set_dtend (vfb, icaltime_from_timet_with_zone (end, FALSE, utc_zone));
 
 	/* add all objects in the given interval */
-	query = g_strdup_printf ("occur-in-time-range? %lu %lu", start, end);
+	iso_start = isodate_from_time_t (start);
+	iso_end = isodate_from_time_t (end);
+	query = g_strdup_printf ("occur-in-time-range? (make-time \"%s\") (make-time \"%s\")",
+				 iso_start, iso_end);
 	obj_sexp = e_cal_backend_sexp_new (query);
 	g_free (query);
+	g_free (iso_start);
+	g_free (iso_end);
 
 	if (!obj_sexp)
 		return vfb;
@@ -1577,7 +1583,6 @@ e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, const
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
 	icalcomponent *icalcomp;
-	icalcomponent_kind kind;
 	ECalComponent *comp;
 	const char *comp_uid;
 	struct icaltimetype current;
@@ -1588,13 +1593,13 @@ e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, const
 	g_return_val_if_fail (priv->icalcomp != NULL, GNOME_Evolution_Calendar_NoSuchCal);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_ObjectNotFound);
 
+	/* Parse the icalendar text */
 	icalcomp = icalparser_parse_string ((char *) calobj);
 	if (!icalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
 
-	/* FIXME Check kind with the parent */
-	kind = icalcomponent_isa (icalcomp);
-	if (kind != ICAL_VEVENT_COMPONENT && kind != ICAL_VTODO_COMPONENT) {
+	/* Check kind with the parent */
+	if (icalcomponent_isa (icalcomp) != e_cal_backend_get_kind (E_CAL_BACKEND (backend))) {
 		icalcomponent_free (icalcomp);
 		return GNOME_Evolution_Calendar_InvalidObject;
 	}
@@ -1640,7 +1645,6 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
 	icalcomponent *icalcomp;
-	icalcomponent_kind kind;
 	const char *comp_uid, *rid;
 	char *real_rid;
 	ECalComponent *comp, *recurrence;
@@ -1653,13 +1657,13 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	g_return_val_if_fail (priv->icalcomp != NULL, GNOME_Evolution_Calendar_NoSuchCal);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_ObjectNotFound);
 
+	/* Parse the icalendar text */
 	icalcomp = icalparser_parse_string ((char *) calobj);
 	if (!icalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
 
-	/* check kind with the parent */
-	kind = icalcomponent_isa (icalcomp);
-	if (kind != ICAL_VEVENT_COMPONENT && kind != ICAL_VTODO_COMPONENT) {
+	/* Check kind with the parent */
+	if (icalcomponent_isa (icalcomp) != e_cal_backend_get_kind (E_CAL_BACKEND (backend))) {
 		icalcomponent_free (icalcomp);
 		return GNOME_Evolution_Calendar_InvalidObject;
 	}
