@@ -166,7 +166,7 @@ e_cal_backend_http_get_alarm_email_address (ECalBackendSync *backend, EDataCal *
 static ECalBackendSyncStatus
 e_cal_backend_http_get_static_capabilities (ECalBackendSync *backend, EDataCal *cal, char **capabilities)
 {
-	*capabilities = CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS;
+	*capabilities = g_strdup (CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS);
 	
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -365,7 +365,8 @@ begin_retrieval_cb (ECalBackendHttp *cbhttp)
 
 /* Open handler for the file backend */
 static ECalBackendSyncStatus
-e_cal_backend_http_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists)
+e_cal_backend_http_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
+			 const char *username, const char *password)
 {
 	ECalBackendHttp *cbhttp;
 	ECalBackendHttpPrivate *priv;
@@ -393,7 +394,7 @@ e_cal_backend_http_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_
 		g_idle_add ((GSourceFunc) begin_retrieval_cb, cbhttp);
 	}
 
-	e_cal_backend_sync_open (priv->file_backend, cal, FALSE);
+	e_cal_backend_sync_open (priv->file_backend, cal, FALSE, username, password);
 	return GNOME_Evolution_Calendar_Success;
 }
 
